// -*- C++ -*-
//
//  This file is part of CBCBSIM, the Combined Broadcast Content-Based
//  (CBCB) routing simulation.
//
//  Authors: See the file AUTHORS for full details. 
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: cbcbsim.cc,v 1.1 2004/03/19 17:04:47 carzanig Exp $
//
#include <cassert>
#include <iostream>
#include <cbnsim/argument_exception.h>
#include "cbcbsim/CBCBSimulation.h"

using namespace cbcbsim;
using namespace cbnsim;
using namespace ssim;
using namespace std;

void parse_args( CBCBSimulation& sim, int argc, char** argv ) throw ( argument_exception );
void print_usage();
void print_version();

int main( int argc, char** argv )
{
  try
    {
      CBCBSimulation sim;

      parse_args( sim, argc, argv );

      sim.run();
    }
  catch( argument_exception& e )
    {
      cerr << "INVALID ARGUMENT: " << e.what() << endl;
      print_usage();
      return 1;
    }
  catch( exception& e )
    {
      cerr << "EXCEPTION: " << e.what() << endl;
      return 1;
    }
  return 0;
}

void parse_args( CBCBSimulation& sim, int argc, char** argv ) throw ( argument_exception )
{
  for( int i = 1; i < argc; ++i )
    {
      if( !strcmp( argv[i], "-d" ) )
	{
	  sim.debug_messages( true );
	}
      else if( !strcmp( argv[i], "-dci" ) )
	{
	  if( ++i == argc )
	    {
	      throw argument_exception( "command-line argument '-dci' expects an argument" );
	    }
	  sim.dci( atof( argv[i] ) );
	}
      else if( !strcmp( argv[i], "-h" ) )
	{
	  print_usage();
	  exit( 0 );
	}
      else if( !strcmp( argv[i], "-i" ) )
	{
	  sim.info_messages( true );
	}
      else if( !strcmp( argv[i], "-o" ) )
	{
	  if( ++i == argc )
	    {
	      throw argument_exception( "command-line argument '-o' expects an argument" );
	    }
	  sim.output_fname( argv[i] );
	}
      else if( !strcmp( argv[i], "-pip" ) )
	{
	  if( ++i == argc )
	    {
	      throw argument_exception( "command-line argument '-pip' expects an argument" );
	    }
	  if( argv[i][0] == 'i' )
	    {
	      sim.pip( ProtocolOptions::PIP_IMMEDIATE );
	      sim.pip_sec( 0 );
	    }
	  else if( argv[i][0] == 't' )
	    {
	      if( ++i == argc )
		{
		  throw argument_exception( "command-line argument '-pip t' expects an argument" );
		}
	      sim.pip( ProtocolOptions::PIP_TIMED );
	      sim.pip_sec( atoi( argv[i] ) );
	    }
	  else
	    {
	      throw argument_exception( string( "unrecognized -pip flag: " + string( argv[i] ) ) );
	    }
	}
      else if( !strcmp( argv[i], "-prp" ) )
	{
	  if( ++i == argc )
	    {
	      throw argument_exception( "command-line argument '-prp' expects an argument" );
	    }
	  if( argv[i][0] == 'n' )
	    {
	      sim.prp( ProtocolOptions::PRP_NEVER );
	      sim.prp_sec( 0 );
	    }
	  else if( argv[i][0] == 't' )
	    {
	      if( ++i == argc )
		{
		  throw argument_exception( "command-line argument '-prp t' expects an argument" );
		}
	      sim.prp( ProtocolOptions::PRP_TIMED );
	      sim.prp_sec( atoi( argv[i] ) );
	    }
	  else
	    {
	      throw argument_exception( string( "unrecognized -prp flag: " ) + string( argv[i] ) );
	    }
	}
      else if( !strcmp( argv[i], "-srsp" ) )
	{
	  if( ++i == argc )
	    {
	      throw argument_exception( "command-line argument '-srsp' expects an argument" );
	    }
	  if( argv[i][0] == 'n' )
	    {
	      sim.srsp( ProtocolOptions::SRSP_NEVER );
	      sim.srsp_count( 0 );
	      sim.srsp_sec( 0 );
	    }
	  else if( argv[i][0] == 't' )
	    {
	      if( ++i == argc )
		{
		  throw argument_exception( "command-line argument '-srsp t' expects an argument" );
		}
	      sim.srsp( ProtocolOptions::SRSP_TIMED );
	      sim.srsp_count( 0 );
	      sim.srsp_sec( atoi( argv[i] ) );
	    }
	  else if( argv[i][0] == 'c' )
	    {
	      if( ++i == argc )
		{
		  throw argument_exception( "command-line argument '-srsp c' expects two arguments" );
		}
	      sim.srsp( ProtocolOptions::SRSP_TIMED_COUNT );
	      sim.srsp_sec( atoi( argv[i] ) );
	      if( ++i == argc )
		{
		  throw argument_exception( "command-line argument '-srsp c' expects two arguments" );
		}
	      sim.srsp_count( atoi( argv[i] ) );
	    }
	  else
	    {
	      throw argument_exception( string( "unrecognized -srsp flag: " ) + string( argv[i] ) );
	    }
	}
      else if( !strcmp( argv[i], "-v" ) )
	{
	  print_version();
	  exit( 0 );
	}
      else if( !strcmp( argv[i], "-w" ) )
	{
	  if( ++i == argc )
	    {
	      throw argument_exception( "command-line argument '-w' expects an argument" );
	    }
	  sim.input_fname( argv[i] );
	}
      else
	{
	  throw argument_exception( string( "unrecognized command-line argument: " ) + string( argv[i] ) );
	}
    }
}

void print_usage()
{
  cerr << "cbcbsim [options]" << endl << endl
       << " -d              turn debug messages on." << endl
       << " -dci <sec>      data collection interval." << endl
       << " -h              print this help message." << endl
       << " -i              turn info messages on." << endl
       << " -o <file>       print output to this file. default is stdout." << endl
       << " -pip  i         immediate predicate issue policy." << endl
       << "       t <sec>   timed predicate issue policy." << endl
       << "       h <sec>   hybrid: immediate but not more than one per <sec> window" << endl
       << " -ppd <num>      probability of a packed being dropped by a link" << endl
       << " -prp  n         never predicate refresh policy." << endl
       << "       t <sec>   timed predicate refresh policy." << endl
       << " -rapp f	    filtered RA propagation policy." << endl
       << "       c <num>   counted random-relax RA propagation policy." << endl
       << " -srsp n         never SR send policy." << endl
       << "       t <sec>   timed SR send policy." << endl
       << "       c <sec> <num>   timed-count SR send policy." << endl
       << " -v              print the program version." << endl
       << " -w <file>       use the specified workload file. default is stdin." << endl
       << endl;
}

void print_version()
{
  cout << "cbcbsim v" << VERSION
       << " (compiled: " << __DATE__
       << " " << __TIME__
       << ")"
       << endl;
}

